@@ -67,12 +67,14 @@ XtActionsRec actions[] = {
 	{ "Execute",		(XtActionProc)Execute_action },
 	{ "Expect",		(XtActionProc)Expect_action },
 	{ "FieldEnd",		(XtActionProc)FieldEnd_action },
+	{ "FieldExit",		(XtActionProc)FieldExit_action },
 	{ "FieldMark",		(XtActionProc)FieldMark_action },
 	{ "Flip",		(XtActionProc)Flip_action },
 	{ "FocusEvent",		(XtActionProc)FocusEvent_action },
 	{ "GraphicsExpose",	(XtActionProc)GraphicsExpose_action },
 	{ "HandleMenu",		(XtActionProc)HandleMenu_action },
 	{ "HardPrint",		(XtActionProc)PrintText_action },
+	{ "HexString",		(XtActionProc)HexString_action },
 	{ "Home",		(XtActionProc)Home_action },
 	{ "Info",		(XtActionProc)Info_action },
 	{ "Insert",		(XtActionProc)Insert_action },
