@@ -106,6 +106,9 @@ static struct ta {
 } *ta_head = (struct ta *) NULL,
   *ta_tail = (struct ta *) NULL;
 
+static char dxl[] = "0123456789abcdef";
+#define FROM_HEX(c)	(strchr(dxl, tolower(c)) - dxl)
+
 
 /*
  * Put an action on the typeahead queue.
@@ -2043,6 +2046,54 @@ Cardinal *num_params;
 	cursor_move(baddr);
 }
 
+/*
+ * FieldExit for the 5250-like emulation.
+ * Erases from the current cursor position to the end of the field, and moves
+ * the cursor to the beginning of the next field.
+ *
+ * Derived from work (C) Minolta (Schweiz) AG, Beat Rubischon <bru@minolta.ch>
+ */
+/*ARGSUSED*/
+void
+FieldExit_action(w, event, params, num_params)
+Widget w;
+XEvent *event;
+String *params;
+Cardinal *num_params;
+{
+	register int    baddr;
+	register unsigned char  *fa;
+
+	action_debug(FieldExit_action, event, params, num_params);
+	if (IN_ANSI) {
+		net_sendc('\n');
+		return;
+	}
+	if (kybdlock) {
+		enq_ta(FieldExit_action, CN, CN);
+		return;
+	}
+	baddr = cursor_addr;
+	fa = get_field_attribute(baddr);
+	if (FA_IS_PROTECTED(*fa) || IS_FA(screen_buf[baddr])) {
+		operator_error(KL_OERR_PROTECTED);
+		return;
+	}
+	if (formatted) {        /* erase to next field attribute */
+		do {
+			  ctlr_add(baddr, CG_null, 0);
+			  INC_BA(baddr);
+		} while (!IS_FA(screen_buf[baddr]));
+		mdt_set(fa);
+		cursor_move(next_unprotected(cursor_addr));
+	} else {        /* erase to end of screen */
+		do {
+			ctlr_add(baddr, CG_null, 0);
+			INC_BA(baddr);
+		} while (baddr != 0);
+	}
+}
+
 
 /*
  * X-dependent code starts here.
@@ -2415,7 +2466,49 @@ Cardinal *num_params;
 		(void) strcat(s, params[i]);
 
 	/* Set a pending string. */
-	ps_set(s);
+	ps_set(s, False);
+}
+
+/*
+ * HexString action.
+ */
+/*ARGSUSED*/
+void
+HexString_action(w, event, params, num_params)
+Widget w;
+XEvent *event;
+String *params;
+Cardinal *num_params;
+{
+	int i;
+	int len = 0;
+	char *s;
+	char *t;
+
+	action_debug(HexString_action, event, params, num_params);
+
+	/* Determine the total length of the strings. */
+	for (i = 0; i < *num_params; i++) {
+		t = params[i];
+		if (!strncmp(t, "0x", 2) || !strncmp(t, "0X", 2))
+			t += 2;
+		len += strlen(t);
+	}
+	if (!len)
+		return;
+
+	/* Allocate a block of memory and copy them in. */
+	s = XtMalloc(len + 1);
+	*s = '\0';
+	for (i = 0; i < *num_params; i++) {
+		t = params[i];
+		if (!strncmp(t, "0x", 2) || !strncmp(t, "0X", 2))
+			t += 2;
+		(void) strcat(s, t);
+	}
+
+	/* Set a pending string. */
+	ps_set(s, True);
 }
 
 /*
@@ -2903,6 +2996,89 @@ Cardinal *num_params;
 	}
 }
 
+/*
+ * Pretend that a sequence of hexadecimal characters was entered at the
+ * keyboard.  The input is a sequence of hexadecimal bytes, 2 characters
+ * per byte.  If connected in ANSI mode, these are treated as ASCII
+ * characters; if in 3270 mode, they are considered EBCDIC.
+ *
+ * Graphic Escapes are handled as \E.
+ */
+void
+hex_input(s)
+char *s;
+{
+	char *t;
+	Boolean escaped;
+	unsigned char *xbuf;
+	unsigned char *tbuf;
+	int nbytes = 0;
+
+	/* Validate the string. */
+	if (strlen(s) % 2) {
+		popup_an_error("%s: Odd number of characters in specification",
+		    action_name(HexString_action));
+		return;
+	}
+	t = s;
+	escaped = False;
+	while (*t) {
+		if (isxdigit(*t) && isxdigit(*(t + 1))) {
+			escaped = False;
+			nbytes++;
+		} else if (!strncmp(t, "\\E", 2) || !strncmp(t, "\\e", 2)) {
+			if (escaped) {
+				popup_an_error("%s: Double \\E",
+				    action_name(HexString_action));
+				return;
+			}
+			if (!IN_3270) {
+				popup_an_error("%s: \\E in ANSI mode",
+				    action_name(HexString_action));
+				return;
+			}
+			escaped = True;
+		} else {
+			popup_an_error("%s: Illegal character in specification",
+			    action_name(HexString_action));
+			return;
+		}
+		t += 2;
+	}
+	if (escaped) {
+		popup_an_error("%s: Nothing follows \\E",
+		    action_name(HexString_action));
+		return;
+	}
+
+	/* Allocate a temporary buffer. */
+	if (!IN_3270 && nbytes)
+		tbuf = xbuf = (unsigned char *)XtMalloc(nbytes);
+
+	/* Pump it in. */
+	t = s;
+	escaped = False;
+	while (*t) {
+		if (isxdigit(*t) && isxdigit(*(t + 1))) {
+			unsigned c;
+
+			c = (FROM_HEX(*t) * 16) + FROM_HEX(*(t + 1));
+			if (IN_3270)
+				key_Character(ebc2cg[c], escaped);
+			else
+				*tbuf++ = (unsigned char)c;
+			escaped = False;
+		} else if (!strncmp(t, "\\E", 2) || !strncmp(t, "\\e", 2)) {
+			escaped = True;
+		}
+		t += 2;
+	}
+	if (!IN_3270 && nbytes) {
+		net_hexansi_out(xbuf, nbytes);
+		XtFree((XtPointer)xbuf);
+	}
+}
+
 /*ARGSUSED*/
 void
 ignore_action(w, event, params, num_params)
