@@ -46,8 +46,10 @@ extern void EraseEOF_action();
 extern void EraseInput_action();
 extern void Erase_action();
 extern void FieldEnd_action();
+extern void FieldExit_action();
 extern void FieldMark_action();
 extern void Flip_action();
+extern void HexString_action();
 extern void Home_action();
 extern void ignore_action();
 extern void Insert_action();
