@@ -61,6 +61,8 @@ typedef struct sms {
 		SS_KBWAIT,	/* command awaiting keyboard unlock */
 		SS_CONNECT_WAIT,/* command awaiting connection to complete */
 		SS_PAUSED,	/* stopped in PauseScript action */
+		SS_WAIT_ANSI,	/* awaiting completeion of Wait(ansi) */
+		SS_WAIT_3270,	/* awaiting completeion of Wait(3270) */
 		SS_WAIT,	/* awaiting completion of Wait() */
 		SS_EXPECTING,	/* awaiting completion of Expect() */
 		SS_CLOSING	/* awaiting completion of Close() */
@@ -74,6 +76,7 @@ typedef struct sms {
 	Boolean	success;
 	Boolean	need_prompt;
 	Boolean	is_login;
+	Boolean is_hex;
 	FILE   *outfile;
 	int	infd;
 	int	pid;
@@ -618,16 +621,30 @@ run_string()
 	if (toggled(DS_TRACE))
 		(void) fprintf(tracef, "String[%d]: '%s'\n", sms_depth,
 		    sms->dptr);
-	if ((len_left = emulate_input(sms->dptr, len, False))) {
-		sms->dptr += len - len_left;
+	if (sms->is_hex) {
 		if (KBWAIT) {
 			sms->state = SS_KBWAIT;
 			if (toggled(DS_TRACE))
-				(void) fprintf(tracef, "%s[%d] paused %d\n",
+				(void) fprintf(tracef,
+				    "%s[%d] paused %d\n",
 				    ST_NAME, sms_depth, sms->state);
+		} else {
+			hex_input(sms->dptr);
+			sms_pop(False);
 		}
 	} else {
-		sms_pop(False);
+		if ((len_left = emulate_input(sms->dptr, len, False))) {
+			sms->dptr += len - len_left;
+			if (KBWAIT) {
+				sms->state = SS_KBWAIT;
+				if (toggled(DS_TRACE))
+					(void) fprintf(tracef,
+					    "%s[%d] paused %d\n",
+					    ST_NAME, sms_depth, sms->state);
+			}
+		} else {
+			sms_pop(False);
+		}
 	}
 }
 
@@ -734,9 +751,10 @@ Boolean is_login;
 
 /* Push a string on the stack. */
 static void
-push_string(s, is_login)
+push_string(s, is_login, is_hex)
 char *s;
 Boolean is_login;
+Boolean is_hex;
 {
 	if (!sms_push(ST_STRING))
 		return;
@@ -750,16 +768,18 @@ Boolean is_login;
 		sms->is_login = True;
 	} else
 		sms->state = SS_INCOMPLETE;
+	sms->is_hex = is_hex;
 	if (sms_depth == 1)
 		sms_continue();
 }
 
 /* Set a pending string. */
 void
-ps_set(s)
+ps_set(s, is_hex)
 char *s;
+Boolean is_hex;
 {
-	push_string(s, False);
+	push_string(s, False, is_hex);
 }
 
 /* Callback for macros menu. */
@@ -963,6 +983,20 @@ sms_continue()
 				return;
 			break;
 
+		    case SS_WAIT_ANSI:
+			if (IN_ANSI) {
+				sms->state = SS_WAIT;
+				continue;
+			}
+			return;
+
+		    case SS_WAIT_3270:
+			if (IN_3270) {
+				sms->state = SS_WAIT;
+				continue;
+			}
+			return;
+
 		    case SS_WAIT:
 			if (!CAN_PROCEED)
 				return;
@@ -1286,24 +1320,39 @@ XEvent *event;
 String *params;
 Cardinal *num_params;
 {
-	if (check_usage(Wait_action, *num_params, 0, 0) < 0)
+	enum sms_state next_state = SS_WAIT;
+
+	if (check_usage(Wait_action, *num_params, 0, 1) < 0)
 		return;
 	if (sms == SN || sms->state != SS_RUNNING) {
-		popup_an_error("%s: can only be called from scripts",
+		popup_an_error("%s: can only be called from scripts or macros",
 		    action_name(Wait_action));
 		return;
 	}
+	if (*num_params == 1) {
+		if (!strcmp(params[0], "ansi")) {
+			if (!IN_ANSI)
+				next_state = SS_WAIT_ANSI;
+		} else if (!strcmp(params[0], "3270")) {
+			if (!IN_3270)
+				next_state = SS_WAIT_3270;
+		} else {
+			popup_an_error("%s: parameter must be 'ansi' or '3270'",
+			    action_name(Wait_action));
+			return;
+		}
+	}
 	if (!(CONNECTED || HALF_CONNECTED)) {
 		popup_an_error("%s: not connected", action_name(Wait_action));
 		return;
 	}
 
 	/* Is it already okay? */
-	if (CAN_PROCEED)
+	if (next_state == SS_WAIT && CAN_PROCEED)
 		return;
 
 	/* No, wait for it to happen. */
-	sms->state = SS_WAIT;
+	sms->state = next_state;
 }
 
 /*
