@@ -1551,7 +1551,7 @@ hostfile_init()
 		struct host *h;
 		char *slash;
 
-		if (strlen(buf) > (unsigned) 1)
+		if (strlen(buf) > (unsigned)1 && buf[strlen(buf) - 1] == '\n')
 			buf[strlen(buf) - 1] = '\0';
 		while (isspace(*s))
 			s++;
