@@ -841,7 +841,7 @@ Boolean *ge;
 		    case 0:
 		    default:
 			return cg2asc[screen_buf[baddr]];
-		    case 1:
+		    case CS_GE:
 			if (screen_buf[baddr] == CG_Yacute)
 				return '[';
 			else if (screen_buf[baddr] == CG_diaeresis)
