@@ -627,8 +627,9 @@ unsigned char c;
 				(void) sprintf(trace_msg, "SENT %s %s\n",
 				    cmd(DO), opt(c));
 				trace_str(trace_msg);
+				check_in3270();
+				check_linemode(False);
 			}
-			check_in3270();
 			break;
 		    default:
 			dont_opt[2] = c;
@@ -639,30 +640,21 @@ unsigned char c;
 			break;
 		}
 		telnet_state = TNS_DATA;
-		check_linemode(False);
 		break;
 	    case TNS_WONT:	/* telnet WONT DO OPTION command */
 		trace_str(opt(c));
 		trace_str("\n");
-		switch (c) {
-		    case TELOPT_BINARY:
-		    case TELOPT_EOR:
-		    case TELOPT_TTYPE:
-			if (hisopts[c]) {
-				dont_opt[2] = c;
-				net_rawout(dont_opt, sizeof(dont_opt));
-				(void) sprintf(trace_msg, "SENT %s %s\n",
-				    cmd(DONT), opt(c));
-				trace_str(trace_msg);
-			}
-			/* fall through */
-		    default:
+		if (hisopts[c]) {
 			hisopts[c] = 0;
-			break;
+			dont_opt[2] = c;
+			net_rawout(dont_opt, sizeof(dont_opt));
+			(void) sprintf(trace_msg, "SENT %s %s\n",
+			    cmd(DONT), opt(c));
+			trace_str(trace_msg);
+			check_in3270();
+			check_linemode(False);
 		}
 		telnet_state = TNS_DATA;
-		check_in3270();
-		check_linemode(False);
 		break;
 	    case TNS_DO:	/* telnet PLEASE DO OPTION command */
 		trace_str(opt(c));
@@ -675,14 +667,16 @@ unsigned char c;
 		    case TELOPT_NAWS:
 		    case TELOPT_TM:
 			if (!myopts[c]) {
-				myopts[c] = 1;
+				if (c != TELOPT_TM)
+					myopts[c] = 1;
 				will_opt[2] = c;
 				net_rawout(will_opt, sizeof(will_opt));
 				(void) sprintf(trace_msg, "SENT %s %s\n",
 				    cmd(WILL), opt(c));
 				trace_str(trace_msg);
+				check_in3270();
+				check_linemode(False);
 			}
-			check_in3270();
 			if (c == TELOPT_NAWS)
 				send_naws();
 			break;
@@ -699,19 +693,15 @@ unsigned char c;
 	    case TNS_DONT:	/* telnet PLEASE DON'T DO OPTION command */
 		trace_str(opt(c));
 		trace_str("\n");
-		switch (c) {
-		    case TELOPT_BINARY:
-		    case TELOPT_EOR:
-		    case TELOPT_TTYPE:
+		if (myopts[c]) {
+			myopts[c] = 0;
 			wont_opt[2] = c;
 			net_rawout(wont_opt, sizeof(wont_opt));
 			(void) sprintf(trace_msg, "SENT %s %s\n",
 			    cmd(WONT), opt(c));
 			trace_str(trace_msg);
-			/* fall through */
-		    default:
-			myopts[c] = 0;
-			break;
+			check_in3270();
+			check_linemode(False);
 		}
 		telnet_state = TNS_DATA;
 		break;
@@ -745,7 +735,6 @@ unsigned char c;
 				    cmd(SB), opt(TELOPT_TTYPE),
 				    telquals[TELQUAL_IS], termtype, cmd(SE));
 				trace_str(trace_msg);
-				check_in3270();
 			}
 		} else {
 			*sbptr = c;	/* just stuff it */
@@ -842,6 +831,50 @@ int	len;
 	}
 }
 
+/*
+ * net_hexansi_out
+ *	Send uncontrolled user data to the host in ANSI mode, performing IAC
+ *	and CR quoting as necessary.
+ */
+void
+net_hexansi_out(buf, len)
+unsigned char *buf;
+int len;
+{
+	unsigned char *tbuf;
+	unsigned char *xbuf;
+
+	if (!len)
+		return;
+
+	/* Trace the data. */
+	if (toggled(DS_TRACE)) {
+		int i;
+
+		(void) fprintf(tracef, ">");
+		for (i = 0; i < len; i++)
+			(void) fprintf(tracef, " %s", ctl_see((int) *(buf+i)));
+		(void) fprintf(tracef, "\n");
+	}
+
+	/* Expand it. */
+	tbuf = xbuf = (unsigned char *)XtMalloc(2*len);
+	while (len) {
+		unsigned char c = *buf++;
+
+		*tbuf++ = c;
+		len--;
+		if (c == IAC)
+			*tbuf++ = IAC;
+		else if (c == '\r' && (!len || *buf != '\n'))
+			*tbuf++ = '\0';
+	}
+
+	/* Send it to the host. */
+	net_rawout(xbuf, tbuf - xbuf);
+	XtFree(xbuf);
+}
+
 
 /*
  * net_cookedout
@@ -1408,7 +1441,7 @@ net_sendc(c)
 char	c;
 {
 	if (c == '\r' && !linemode)	/* CR must be quoted */
-		net_cookout("\r\n", 2);
+		net_cookout("\r\0", 2);
 	else
 		net_cookout(&c, 1);
 }
